@@ -2,6 +2,7 @@
 
 import argparse
 
+from soco_cli.check_for_update import print_update_status
 from soco_cli.speakers import Speakers
 from soco_cli.utils import (
     check_args,
@@ -53,6 +54,10 @@ def main():
         docs()
         exit(0)
 
+    if args.check_for_update:
+        print_update_status()
+        exit(0)
+
     configure_logging(args.log)
 
     # Create the Speakers object
